﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(PlayerController))]
public class TakeCheese : MonoBehaviour 
{
    //チーズタグの名前
    [SerializeField]
    string cheeseTagName = "Cheese";

    //チーズが取れる範囲
    [SerializeField]
    float rangeTake = 1.0f;

    //チーズをどのくらい前に置くか
    [SerializeField]
    float distancePut = 0.9f;

    //チーズが取れる正面ベクトルからの角度(degree)
    [SerializeField]
    float takeDeg = 45.0f;

    //チーズを取るときに押すボタン
    [SerializeField]
    KeyCode takeCheeseKey = KeyCode.Space;

    //チーズを持っているときにどのくらい遅くするか
    [SerializeField,Range(0,1)]
    float slowLevel = 0.5f;

    PlayerController playerController;

    //プレイヤーのデフォルトのスピード
    float startSpeed;

    //チーズを持っているか
    private bool isTakeCheese = false;

    //持っているチーズ
    private Cheese takenCheese = null;

    //自分自身のトランスフォーム
    private Transform ownTrans;

	// Use this for initialization
	void Start () 
    {
        ownTrans = GetComponent<Transform>();

        playerController = GetComponent<PlayerController>();
        startSpeed = playerController.maximumSpeed;
	}
	
	// Update is called once per frame
	void Update () 
    {
		if(Input.GetKeyDown(takeCheeseKey))
        {
            if(!isTakeCheese)
            {
                PickCheese();
            }
            else
            {
                PutCheese();
            }
        }
	}

    /// <summary>
    /// チーズを取る関数
    /// </summary>
    void PickCheese()
    {
        GameObject[] cheeses = GameObject.FindGameObjectsWithTag(cheeseTagName);
        SortByDistance(ref cheeses, ownTrans.position);

        foreach(GameObject cheese in cheeses)
        {
            Vector3 dir = cheese.GetComponent<Transform>().position - ownTrans.position;
            float distance = dir.magnitude;

            if (!(rangeTake > 0 && distance <= rangeTake))
                continue;

            float deg = Vector3.Angle(ownTrans.forward, dir.normalized);

            if(deg <= takeDeg)
            {
                Cheese cheeseSrc = cheese.GetComponent<Cheese>();

                if(cheeseSrc)
                {
                    cheeseSrc.Take(ownTrans);

                    isTakeCheese = true;
                    PlayerSlowSpeed();

                    takenCheese = cheeseSrc;

                    break;
                }
            }
            else
            {
                Debug.LogWarning(cheese.name + "doesn't set Cheese script");
            }
        }
    }

    /// <summary>
    /// チーズを置く関数
    /// </summary>
    void PutCheese()
    {
        if (takenCheese == null)
            return;

        Vector3 putPos = ownTrans.position + ownTrans.forward * distancePut;
        putPos.y = takenCheese.StartPosY;

        takenCheese.Put(putPos);

        GameObject box = GameObject.FindGameObjectWithTag("CheeseBase");
        CheeseBase cheeseBase = null;
        if (box)
            cheeseBase = box.GetComponent<CheeseBase>();

        if (cheeseBase)
        {
            if (CheckCheeseIsInBox(putPos))
            {
                if (!takenCheese.IsInBox)
                {
                    cheeseBase.AddCheeseCount();
                    takenCheese.IsInBox = true;
                }
            }
            else
            {
                //領域から離れておかれた場合
                //入ったチーズの数をマイナス１する
                if (takenCheese.IsInBox)
                {
                    cheeseBase.MinusCheeseCount();
                    takenCheese.IsInBox = false;
                }
            }
        }

        Debug.Log("CheeseCount is " + cheeseBase.CheeseCount.ToString());

        takenCheese = null;
        isTakeCheese = false;
        PlayerNormalSpeed();
    }

    /// <summary>
    /// チーズが領域に入ったかどうか判定する
    /// </summary>
    /// <param name="putPos">チーズが置かれる場所</param>
    /// <returns></returns>
    bool CheckCheeseIsInBox(Vector3 putPos)
    {
        GameObject box = GameObject.FindGameObjectWithTag("CheeseBase");

        if (box == null)
            return false;

        CheeseBase cheeseBase = box.GetComponent<CheeseBase>();
        if (cheeseBase == null)
            return false;

        if (takenCheese == null)
            return false;

        GameObject cheese = takenCheese.gameObject;

        Transform boxTrans = box.GetComponent<Transform>();

        float distance = (boxTrans.position - putPos).magnitude;
        if(distance <= cheeseBase.CheeseAcceptDistance)
        {
            return true;
        }

        return false;
    }

    /// <summary>
    /// プレイヤーのスピードを遅くする
    /// </summary>
    void PlayerSlowSpeed()
    {
        playerController.maximumSpeed = startSpeed * slowLevel;
        playerController.minimumSpeed = -startSpeed * slowLevel;
    }

    /// <summary>
    /// プレイヤーのスピードをもとに戻す
    /// </summary>
    void PlayerNormalSpeed()
    {
        playerController.maximumSpeed = startSpeed;
        playerController.minimumSpeed = -startSpeed;
    }

    /// <summary>
    /// 指定された位置からの距離で昇順に並べる
    /// </summary>
    /// <param name="target">並べ替える対象</param>
    /// <param name="playerPos">基準の位置</param>
    void SortByDistance(ref GameObject[] target,Vector3 playerPos)
    {
        GameObject tmp;
        float aDistance,bDistance;
        Vector3 aPos, bPos;

        for(int i = 0; i < target.Length - 1; i++)
        {
            aPos = target[i].GetComponent<Transform>().position;
            aDistance = (playerPos - aPos).magnitude;

            for(int j = i + 1; j < target.Length; j++)
            {
                bPos = target[j].GetComponent<Transform>().position;
                bDistance = (playerPos - bPos).magnitude;

                if(aDistance > bDistance)
                {
                    tmp = target[i];
                    target[i] = target[j];
                    target[j] = tmp;
                }
                
            }
        }
    }
}
